/**
 * OW2 Util
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SshProtocolConfigFactory.java 5684 2010-11-16 22:10:54Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.protocol.ssh.impl.jsch;

import java.util.Map;

import org.ow2.util.protocol.api.ProtocolException;
import org.ow2.util.protocol.ssh.api.SshProtocolConfig;

/**
 *
 * @author Loris Bouzonnet
 */
public final class SshProtocolConfigFactory {

    public static final String PASSWORD = "password";

    public static final String PRIVATE_KEY_URL = "private-key-url";

    public static final String PASSPHRASE = "passphrase";

    public static final String KNOWN_HOSTS = "known_hosts";

    public static final String HOST = "host";

    public static final String PORT = "port";

    public static final String USER = "user";

    public static final String COMPRESSION = "compression";

    public static final String PROXY_HTTP = "proxy-http";

    public static final String PROXY_SOCKS4 = "proxy-socks4";

    public static final String PROXY_SOCKS5 = "proxy-socks5";

    public static final String X11_FORWARDING = "x11-forwarding";

    private static final String CONNECTION_TIMEOUT = "connection-timeout";

    private static final String CHANNEL_TIMEOUT = "channel-timeout";


    public static SshProtocolConfig createSshProtocolConfig(final String url, final Map<String, String> context) throws ProtocolException {

        final SshProtocolConfig protocolConfig = new SshProtocolConfig();

        final String host;
        final String user;
        final String sPort;

        if (url != null) {

            final String[] userHost = url.split("@");
            final String hostPort;
            if (userHost.length == 1) {
                if (context != null) {
                    user = context.get(USER);
                } else {
                    user = null;
                }
                hostPort = userHost[0];

            } else if (userHost.length == 2) {
                if (context != null && context.get(USER) != null
                        && !context.get(USER).equals(userHost[0])) {
                    throw new ProtocolException();
                }
                user = userHost[0];
                hostPort = userHost[1];

            } else {
                throw new ProtocolException();
            }

            final String[] hostAndPort = hostPort.split(":");
            if (context != null && context.get(HOST) != null
                    && !context.get(HOST).equals(hostAndPort[0])) {
                throw new ProtocolException();
            }
            host = hostAndPort[0];
            if (hostAndPort.length == 2) {
                if (context != null && context.get(PORT) != null
                        && !context.get(PORT).equals(hostAndPort[1])) {
                    throw new ProtocolException();
                }
                sPort = hostAndPort[1];
            } else {
                if (context == null) {
                    sPort = null;
                } else {
                    sPort = context.get(PORT);
                }
            }
        } else {
            if (context == null) {
                throw new ProtocolException("The two arguments are null");
            }
            host = context.get(HOST);
            user = context.get(USER);
            sPort = context.get(PORT);
        }

        protocolConfig.setUser(user);
        protocolConfig.setHost(host);
        if (sPort != null) {
            protocolConfig.setPort(Integer.valueOf(sPort));
        }

        if (context != null) {
        	protocolConfig.setPassword(context.get(PASSWORD));
            protocolConfig.setPrivateKeyPath(context.get(PRIVATE_KEY_URL));
            protocolConfig.setPassphrase(context.get(PASSPHRASE));
            protocolConfig.setKnownHostsPath(context.get(KNOWN_HOSTS));

            final String sLevel = context.get(COMPRESSION);
            if (sLevel != null) {
                protocolConfig.setCompressionLevel(Integer.valueOf(sLevel));
            }
            protocolConfig.setProxyHttp(context.get(PROXY_HTTP));
            protocolConfig.setProxySocks4(context.get(PROXY_SOCKS4));
            protocolConfig.setProxySocks5(context.get(PROXY_SOCKS5));
            protocolConfig.setX11Forwarding(context.get(X11_FORWARDING));

            if (context.containsKey(CONNECTION_TIMEOUT)) {
                protocolConfig.setConnectionTimeout(
                        Integer.valueOf(context.get(CONNECTION_TIMEOUT)));
            }

            if (context.containsKey(CHANNEL_TIMEOUT)) {
                protocolConfig.setChannelTimeout(
                        Integer.valueOf(context.get(CHANNEL_TIMEOUT)));
            }
        }

        return protocolConfig;
    }

}
